import boto3
import time
import logging
import os

logger = logging.getLogger()
try:
    log_level = os.environ["LogLevel"]
    if log_level not in ["INFO", "DEBUG"]:
        log_level = "INFO"
except:
    log_level = "INFO"
logger.setLevel(log_level)

chime = boto3.client("chime")
ssm = boto3.client("ssm")


def createVoiceConnector(
    uid,
    region=None,
    name=None,
    terminationCidrs=None,
    originationIps=None,
    encryption=None,
    streaming=None,
    dataRetention=None,
    notificationTargets=None,
    callingRegions=None,
    **kwargs,
):
    logger.info("Creating a new Voice Connector")
    try:
        voice_connector_id = chime.create_voice_connector(
            Name=name, AwsRegion=region, RequireEncryption=bool(encryption)
        )["VoiceConnector"]["VoiceConnectorId"]
        logger.info(f"Voice Connector: {voice_connector_id}")
    except Exception as e:
        error = {"error": f"Exception thrown: {e}"}
        logger.error(error)
        raise RuntimeError(error)

    try:
        ssm.put_parameter(
            Name="/chime/voiceConnector/" + uid,
            Description="Voice Connector Ordered",
            Overwrite=True,
            Value=voice_connector_id,
            Type="String",
        )

    except Exception as e:
        error = {"error": f"Exception thrown: {e}"}
        logger.error(error)
        raise RuntimeError(error)

    logger.info(f"Streaming: {streaming}")
    streaming_notification_targets = []
    if bool(streaming):
        for notification_target in notificationTargets:
            streaming_notification_targets.append({"NotificationTarget": notification_target})

        logger.info(f"Streaming Notification Targets: {streaming_notification_targets}")
        try:
            chime.put_voice_connector_streaming_configuration(
                VoiceConnectorId=voice_connector_id,
                StreamingConfiguration={
                    "DataRetentionInHours": int(dataRetention),
                    "Disabled": False,
                    "StreamingNotificationTargets": streaming_notification_targets,
                },
            )
        except Exception as e:
            error = {"error": f"Exception thrown: {e}"}
            logger.error(error)
            raise RuntimeError(error)

    logger.info(f"Termination CIDRs: {terminationCidrs}")
    if terminationCidrs:
        try:
            chime.put_voice_connector_termination(
                VoiceConnectorId=voice_connector_id,
                Termination={
                    "CallingRegions": callingRegions,
                    "CidrAllowedList": terminationCidrs,
                    "Disabled": False,
                },
            )
        except Exception as e:
            error = {"error": f"Exception thrown: {e}"}
            logger.error(error)
            raise RuntimeError(error)

    logger.info(f"Origination IPs: {originationIps}")
    if originationIps:
        for originationIp in originationIps:
            try:
                chime.put_voice_connector_origination(
                    VoiceConnectorId=voice_connector_id,
                    Origination={
                        "Routes": [
                            {"Host": originationIp, "Port": 5060, "Protocol": "UDP", "Priority": 1, "Weight": 1}
                        ],
                        "Disabled": False,
                    },
                )
            except Exception as e:
                error = {"error": f"Exception thrown: {e}"}
                logger.error(error)
                raise RuntimeError(error)

    return voice_connector_id


def deleteVoiceConnecytor(uid):
    logger.info(f"Deleting Voice Connector: {uid}")
    try:
        voice_connector_to_delete = ssm.get_parameter(Name="/chime/voiceConnector/" + str(uid))["Parameter"]["Value"]
        logger.info(f"Voice Connector to Delete: {voice_connector_to_delete}")
    except Exception as e:
        error = {"error": f"Exception thrown: {e}"}
        logger.error(error)
        raise RuntimeError(error)

    try:
        associated_phone_numbers = chime.list_phone_numbers(
            FilterName="VoiceConnectorId", FilterValue=voice_connector_to_delete
        )
    except Exception as e:
        error = {"error": f"Exception thrown: {e}"}
        logger.error(error)
        raise RuntimeError(error)

    logger.info(f"Associated Phone Numbers: {associated_phone_numbers}")

    if associated_phone_numbers["PhoneNumbers"]:
        phone_numbers_to_disassociate = []
        for phone_number in associated_phone_numbers["PhoneNumbers"]:
            phone_numbers_to_disassociate.append(phone_number["PhoneNumberId"])
        logger.info(f"Phone Numbers to Disassociate: {phone_numbers_to_disassociate}")
        try:
            chime.disassociate_phone_numbers_from_voice_connector(
                VoiceConnectorId=voice_connector_to_delete, E164PhoneNumbers=phone_numbers_to_disassociate
            )
        except Exception as e:
            error = {"error": f"Exception thrown: {e}"}
            logger.error(error)
            raise RuntimeError(error)

    try:
        chime.delete_voice_connector(VoiceConnectorId=voice_connector_to_delete)
    except Exception as e:
        error = {"error": f"Exception thrown: {e}"}
        logger.error(error)
        raise RuntimeError(error)

    return None
